﻿using AbstractSoftwareInstallationContracts.BusinessLogicsContracts;
using AbstractSoftwareInstallationContracts.BindingModels;
using AbstractSoftwareInstallationContracts.SearchModels;
using AbstractSoftwareInstallationContracts.ViewModels;
using AbstractSoftwareInstallationContracts.StoragesContracts;
using Microsoft.Extensions.Logging;

namespace AbstractSoftwareInstallationBusinessLogic.BusinessLogic
{
    public class PackageLogic : IPackageLogic
    {
        private readonly ILogger _logger;
        private readonly IPackageStorage _packageStorage;
        public PackageLogic(ILogger<PackageLogic> logger, IPackageStorage PackageStorage)
        {
            _logger = logger;
            _packageStorage = PackageStorage;
        }
        public List<PackageViewModel>? ReadList(PackageSearchModel? model)
        {
            _logger.LogInformation("ReadList. PackageName:{PackageName}.Id:{ Id}", model?.PackageName, model?.Id);
            var list = model == null ? _packageStorage.GetFullList() : _packageStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }
        public PackageViewModel? ReadElement(PackageSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. PackageName:{PackageName}.Id:{ Id}", model.PackageName, model.Id);
            var element = _packageStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }
        public bool Create(PackageBindingModel model)
        {
            CheckModel(model);
            if (_packageStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;

        }
        public bool Update(PackageBindingModel model)
        {
            CheckModel(model);
            if (_packageStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }
        public bool Delete(PackageBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id:{Id}", model.Id);
            if (_packageStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }
        private void CheckModel(PackageBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (string.IsNullOrEmpty(model.PackageName))
            {
                throw new ArgumentNullException("Нет названия пакета",
                nameof(model.PackageName));
            }
            if (model.Price <= 0)
            {
                throw new ArgumentNullException("Цена пакета должна быть больше 0", nameof(model.Price));
            }
            _logger.LogInformation("Package. PackageName:{PackageName}.Price:{Price}. Id: {Id}", model.PackageName, model.Price, model.Id);
            var element = _packageStorage.GetElement(new PackageSearchModel
            {
                PackageName = model.PackageName
            });
            if (element != null && element.Id != model.Id)
            {
                throw new InvalidOperationException("Пакет с таким названием уже есть");
            }
        }
    }
    
}
